
From: Tejun Heo <htejun@gmail.com>

When inserting INSERT_BACK request, cfq_insert_request() calls
cfq_dispatch_requests() repetitively until it returns zero indicating no
request is dispatched.  This used to flush all the requests in the queue to
the dispatch queue but, with idle slice implemented, the current active queue
may decide to wait for new request using slice_timer.  When this happens, 0 is
returned from cfq_dispatch_requests() even when other cfqq's have pending
requests.  This breaks INSRET_BACK semantics.

This patch adds @force argument which, when set to non-zero, disables
idle_slice, and uses the argument when flushing cfqq's for INSERT_BACK.  While
at it, use INT_MAX instead of cfq_quantum when flushing cfqq's, as we're gonna
dump all the requests and using cfq_quantum does nothing but adding
unnecessary iterations.

Signed-off-by: Tejun Heo <htejun@gmail.com>
Cc: Jens Axboe <axboe@suse.de>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/block/cfq-iosched.c |   14 ++++++++------
 1 files changed, 8 insertions(+), 6 deletions(-)

diff -puN drivers/block/cfq-iosched.c~cfq-cfq-elevator_insert_back-fix drivers/block/cfq-iosched.c
--- 25/drivers/block/cfq-iosched.c~cfq-cfq-elevator_insert_back-fix	2005-05-25 00:18:07.000000000 -0700
+++ 25-akpm/drivers/block/cfq-iosched.c	2005-05-25 00:18:07.000000000 -0700
@@ -991,7 +991,7 @@ cfq_prio_to_maxrq(struct cfq_data *cfqd,
 /*
  * get next queue for service
  */
-static struct cfq_queue *cfq_select_queue(struct cfq_data *cfqd)
+static struct cfq_queue *cfq_select_queue(struct cfq_data *cfqd, int force)
 {
 	unsigned long now = jiffies;
 	struct cfq_queue *cfqq;
@@ -1012,7 +1012,8 @@ static struct cfq_queue *cfq_select_queu
 	 */
 	if (!RB_EMPTY(&cfqq->sort_list))
 		goto keep_queue;
-	else if (cfq_cfqq_sync(cfqq) && time_before(now, cfqq->slice_end)) {
+	else if (!force && cfq_cfqq_sync(cfqq) &&
+		 time_before(now, cfqq->slice_end)) {
 		if (cfq_arm_slice_timer(cfqd, cfqq))
 			return NULL;
 	}
@@ -1078,7 +1079,8 @@ __cfq_dispatch_requests(struct cfq_data 
 	return dispatched;
 }
 
-static int cfq_dispatch_requests(request_queue_t *q, int max_dispatch)
+static int
+cfq_dispatch_requests(request_queue_t *q, int max_dispatch, int force)
 {
 	struct cfq_data *cfqd = q->elevator->elevator_data;
 	struct cfq_queue *cfqq;
@@ -1086,7 +1088,7 @@ static int cfq_dispatch_requests(request
 	if (!cfqd->busy_queues)
 		return 0;
 
-	cfqq = cfq_select_queue(cfqd);
+	cfqq = cfq_select_queue(cfqd, force);
 	if (cfqq) {
 		cfqq->wait_request = 0;
 		cfqq->must_dispatch = 0;
@@ -1172,7 +1174,7 @@ dispatch:
 		return rq;
 	}
 
-	if (cfq_dispatch_requests(q, cfqd->cfq_quantum))
+	if (cfq_dispatch_requests(q, cfqd->cfq_quantum, 0))
 		goto dispatch;
 
 	return NULL;
@@ -1707,7 +1709,7 @@ cfq_insert_request(request_queue_t *q, s
 
 	switch (where) {
 		case ELEVATOR_INSERT_BACK:
-			while (cfq_dispatch_requests(q, cfqd->cfq_quantum))
+			while (cfq_dispatch_requests(q, INT_MAX, 1))
 				;
 			list_add_tail(&rq->queuelist, &q->queue_head);
 			break;
_
